/**
 * 
 */
package sim.exception;

/**
 * La classe <b>SConstructorException</b> reprsente une exception lance lors de la construction d'un objet
 * lorsqu'il y a un paramtre dcrivant l'objet le rendant invalide.
 * 
 * @author Simon Vzina
 * @since 2015-03-31
 * @version 2022-01-21
 */
public class SConstructorException extends RuntimeException {

  /**
   * La variable <b>serialVersionUID<b> correspond  un code d'identification de l'exception.
   */
  private static final long serialVersionUID = 1344374114943474870L;

  /**
   * La constante <b>DEFAULT_MESSAGE</b> reprsente le message par dfaut lance par une erreur de ce type.
   */
  private static final String DEFAULT_MESSAGE = "Une erreur de construction est survenue.";
  
  /**
   * Constructeur de l'exception avec un message par dfaut.
   */
  public SConstructorException()
  {
    this(DEFAULT_MESSAGE);
  }
  
  /**
   * Constructeur de l'exception avec message d'erreur.
   * 
   * @param message Le message de l'erreur.
   */
  public SConstructorException(String message)
  {
    super(message);
  }

  /**
   * Constructeur de l'exception avec message d'erreur et cause de l'exception.
   * 
   * @param message Le message de l'erreur.
   * @param cause La cause de l'erreur.
   */
  public SConstructorException(String message, Throwable cause)
  {
    super(message, cause);
  }

  /**
   * Constructeur de l'exception avec le message par dfaut et la cause.
   * @param cause
   */
  public SConstructorException(Throwable cause)
  {
    super(DEFAULT_MESSAGE, cause);
  }
  
}
