/**
 * 
 */
package sim.util;

import java.io.IOException;

/**
 * L'interface <b>SReadable</b> permet  l'objet de se configurer  l'aide d'une lecture dans un fichier txt.
 * 
 * @author Simon Vzina
 * @since 2015-01-09
 * @version 2018-01-16
 */
public interface SReadable {

	/**
	 * Mthode qui permet la lecture d'un fichier de format txt pour faire l'initialisation des paramtres de l'objet SReadable.
	 * 
	 * @param sbr Le buffer cherchant l'information dans le fichier txt.
	 * @throws IOException Si une erreur de type I/O est survenue lors de la lecture.
	 * @throws SInitializationException Si une erreur est survenue lors de l'initialisation de l'objet.
	 * @see SBufferedReader
	 */
	public void read(SBufferedReader sbr) throws IOException, SInitializationException;
	
	/**
   * Mthode pour obtenir le nom de l'objet implmentant l'interface <b>SReadable</b>.
   * Ce nom correspond galement au <b>mot cl</b>  rechercher lors d'une lecture avec un <b>SBufferedReader</b> pour en faire la construction de l'objet en question.
   *    
   * @return Le nom de l'objet lisible.
   */
  public String getReadableName();
  
  /**
   * Mthode pour obtenir un tableau des <b>mots cls</b> permettant de dfinir les paramtres de l'objet implmentant l'interface <b>SReadable</b> lors de sa lecture.
   * 
   * @return Un tableau des <b>mots cls</b> des paramtres.
   */
  public String[] getReadableParameterName();
  
}//fin interface SReadable
