package sim.graphics.camera;

import org.junit.Assert;
import org.junit.Test;

import sim.exception.SNoImplementationException;
import sim.math.SVector3d;
import sim.util.SLog;

/**
 * JUnit test permettant de valider les fonctionnalits de la classe <b>SVectorCamera</b>.
 * 
 * @author Simon Vzina
 * @since 2022-05-29
 * @version 2022-05-29
 */
public class SVectorCameraTest {

  /**
   * Test de la mthode <b>rotationYaw</b> dans le cas d'une rotation de 30 degre dans le sens positif, donc vers la droite.
   */
  @Test
  public void rotationYawTest1a() 
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double degrees = 30.0;
            
      SVectorCamera expected = new SVectorCamera(new SVector3d(0.0, 0.0, 0.0), new SVector3d(0.8660254037844387, -0.5, 0.0), SVector3d.UNITARY_Z);
            
      // Mthode  tester.
      camera.rotationYaw(degrees);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SVectorCameraTest ---> Test non effectu : public void rotationYawTest1a()");
    }
  }

  /**
   * Test de la mthode <b>rotationYaw</b> dans le cas d'une rotation de 30 degre dans le sens ngatif, donc vers la gauche.
   */
  @Test
  public void rotationYawTest1b() 
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double degrees = -30.0;
            
      SVectorCamera expected = new SVectorCamera(new SVector3d(0.0, 0.0, 0.0), new SVector3d(0.8660254037844387, 0.5, 0.0), SVector3d.UNITARY_Z);
            
      // Mthode  tester.
      camera.rotationYaw(degrees);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SVectorCameraTest ---> Test non effectu : public void rotationYawTest1b()");
    }
  }

  /**
   * Test de la mthode <b>rotationPitch</b> dans le cas d'une rotation de 30 degre dans le sens positif, donc vers le haut.
   */
  @Test
  public void rotationPitchTest1a() 
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double degrees = 30.0;
            
      SVectorCamera expected = new SVectorCamera(new SVector3d(0.0, 0.0, 0.0), new SVector3d(0.8660254037844387, 0.0, 0.5), SVector3d.UNITARY_Z);
            
      // Mthode  tester.
      camera.rotationPitch(degrees);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SVectorCameraTest ---> Test non effectu : public void rotationPitchTest1a()");
    }
  }
  
  /**
   * Test de la mthode <b>rotationPitch</b> dans le cas d'une rotation de 30 degre dans le sens ngatif, donc vers le bas.
   */
  @Test
  public void rotationPitchTest1b() 
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double degrees = -30.0;
            
      SVectorCamera expected = new SVectorCamera(new SVector3d(0.0, 0.0, 0.0), new SVector3d(0.8660254037844387, 0.0, -0.5), SVector3d.UNITARY_Z);
            
      // Mthode  tester.
      camera.rotationPitch(degrees);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SVectorCameraTest ---> Test non effectu : public void rotationPitchTest1b()");
    }
  }
  
  /**
   * Test de la mthode <b>rotationRoll</b> dans le cas d'une rotation de 30 degre dans le sens positif, donc vers la droite.
   */
  @Test
  public void rotationRollTest1a() 
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double degrees = 30.0;
            
      // Remarque : Tourner du ct droit sera du ct ngatif de l'axe y.
      SVectorCamera expected = new SVectorCamera(new SVector3d(0.0, 0.0, 0.0), SVector3d.UNITARY_X, new SVector3d(0.0, -0.5, 0.8660254037844387));
            
      // Mthode  tester.
      camera.rotationRoll(degrees);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SVectorCameraTest ---> Test non effectu : public void rotationRollTest1a()");
    }
  }
  
  /**
   * Test de la mthode <b>rotationPitch</b> dans le cas d'une rotation de 30 degre dans le sens ngatif, donc vers la gauche.
   */
  @Test
  public void rotationRollTest1b() 
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double degrees = -30.0;
            
      // Remarque : Tourner du ct gauche sera du ct positif de l'axe y.
      SVectorCamera expected = new SVectorCamera(new SVector3d(0.0, 0.0, 0.0), SVector3d.UNITARY_X, new SVector3d(0.0, 0.5, 0.8660254037844387));
            
      // Mthode  tester.
      camera.rotationRoll(degrees);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SVectorCameraTest ---> Test non effectu : public void rotationRollTest1a()");
    }
  }
  
}
